/*
 * low-level API for FmStick
 * $Id$
 * (part of FmStick).
 *
 * (C) 2011 Serge <piratfm@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef FMTX_H
#define FMTX_H

#ifdef FMTX
#define EXTERN
#define ASSIGN (x) =x
#else
#define EXTERN extern
#define ASSIGN(x)
#endif

#ifdef _WIN32
#define FMTX_API_EXPORT __declspec(dllexport)
#define FMTX_API_CALL  _stdcall
#else
#define FMTX_API_EXPORT
#define FMTX_API_CALL
#endif

#include <inttypes.h>

#ifdef __cplusplus
extern "C" {
#endif

//MSB: Command Types
#define PCRequestError      0x80
#define PCTransfer          0x02
//MSB: Request Types
#define RequestDone         0x80

enum {
        RequestNone = 0,
        RequestCpuId,
        RequestSi4711Reset,     	//reset
        RequestSi4711Access,    	//low level access
        RequestSi4711GetProp,   	//medium level get prop
        RequestSi4711SetProp,   	//medium level set prop
        RequestSi4711PowerStatus,
        RequestSi4711PowerUp,   	//high level power up
        RequestSi4711PowerDown, 	//high level power down
        RequestSi4711AudioEnable,       //this MUST BE!!! called after setting config to enable audio.
        RequestSi4711AudioDisable,
        RequestEepromSectionRead,
        RequestEepromSectionWrite,
        RequestSi4711AsqStatus,
        RequestSi4711TuneStatus,
        RequestUnknown
};

enum {
        SI4711_OK = 0,
        SI4711_TIMEOUT,
        SI4711_COMM_ERR,
        SI4711_BAD_ARG,
        SI4711_NOCTS,
        SI4711_ERROR_UNKNOWN
};

/** \brief FMTX mode enumeration. */
typedef enum {
    FMTX_MODE_NONE,            /*!< No FMTX detected. */
    FMTX_MODE_POWER_DOWN,      /*!< FMTX present in power-down mode. */
    FMTX_MODE_POWER_UP,        /*!< FMTX present in power-up mode. */
    FMTX_MODE_TRANSMITTING,    /*!< FMTX present in running mode. */
    FMTX_MODE_OK,              /*!< FMTX present in running mode and command is successfully. */
} FMTX_MODE_ENUM; // The current mode of the FMTX: none inserted, in bootloader mode or in normal application mode

typedef enum {
    FMTX_SPECIAL_FRONTEND,    /*!< Do Stuff on-the-go. */
    FMTX_SPECIAL_EEPROM       /*!< Don't do stuff in-real, just create EEPROM image. */
} FMTX_SPECIAL_ENUM;



/*********************************************************
 * EEPROM Info
 *********************************************************/
#define TX_CONFIG_OFFSET 	0x1E00
#define FMTX_EEPROM_CMDSIZE     0x08
#define FMTX_MAX_EEPROM_CMDS    62

typedef struct {
    uint16_t syncWord;
    uint8_t numCmds;
    uint8_t startupFlag;
    //unused 5 bytes;
    uint8_t commandData[FMTX_MAX_EEPROM_CMDS*FMTX_EEPROM_CMDSIZE];
} eeprom_data;



/*********************************************************
 * Shared Vars
 *********************************************************/
EXTERN FMTX_MODE_ENUM fmtxCmdStatus;
EXTERN FMTX_SPECIAL_ENUM fmtxCmdSpecial;

/*********************************************************
 * High-level API
 *********************************************************/
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxEEPROMInfoClean(void);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxEEPROMReadConfig(void);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxEEPROMWriteConfig(int autorun);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxEEPROMGetStartupFlag(void);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxEEPROMSetStartupFlag(uint8_t flag, int force);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxEEPROMWriteFwChunk(const uint8_t *data, int size, uint16_t offset);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxEEPROMReadFwChunk(uint8_t *data, int size, uint16_t offset);

/*********************************************************
 * Low-level API
 *********************************************************/
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppDeviceFound(void);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppFeReset(void);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppFeGetMode(int *transmitting);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppFeUp(void);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppFeDown(void);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppIdString(char *cpuid, char *rev);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppAsqStatus(uint8_t *flags, int8_t *level);


EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppSetProperty(int16_t i16Prop, int16_t i16Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppGetProperty(int16_t i16Prop, int16_t *pi16Val);

EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppCommand(uint8_t u8Cmd, uint8_t *pu8Resp, uint8_t u8Amount, ...);



/*********************************************************
 * Medium-level API
 *********************************************************/
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioGetAcompFlags();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioSetAcompFlags(uint8_t u8Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL int16_t fmtxAudioGetAcompThreshold();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioSetAcompThreshold(int16_t i16Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL double fmtxAudioGetAcompAttackTime();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioSetAcompAttackTime(double dVal);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioGetAcompReleaseTimeId();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioSetAcompReleaseTimeId(uint8_t u8Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioGetAcompGain();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioSetAcompGain(uint8_t u8Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL double fmtxAudioGetLimiterReleaseTime();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioSetLimiterReleaseTime(double dVal);

/* audio signal measurement */
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioGetAsqIntFlags();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudiosetAsqIntFlags(uint8_t u8Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL int8_t fmtxAudioGetAsqLevelLow();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioSetAsqLevelLow(int8_t i8Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint16_t fmtxAudioGetAsqDurationLow();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioSetAsqDurationLow(uint16_t u16Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL int8_t fmtxAudioGetAsqLevelHigh();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioSetAsqLevelHigh(int8_t i8Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint16_t fmtxAudioGetAsqDurationHigh();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxAudioSetAsqDurationHigh(uint16_t u16Val);

/* digital audio interface */
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppEnableAudio(void);
EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxIoAppDisableAudio(void);


/* transmitting parameters */
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxTransmitterGetComponentFlags();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxTransmitterSetComponentFlags(uint8_t u8Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxTransmitterGetPreemphasisId();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxTransmitterSetPreemphasisId(uint8_t u8Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint32_t fmtxTransmitterGetAudioDeviation();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxTransmitterSetAudioDeviation(uint32_t u32Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint32_t fmtxTransmitterGetPilotDeviation();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxTransmitterSetPilotDeviation(uint32_t u32Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint32_t fmtxTransmitterGetRDSDeviation();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxTransmitterSetRDSDeviation(uint32_t u32Val);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint16_t fmtxTransmitterGetPilotFrequency();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxTransmitterSetPilotFrequency(uint16_t u16Val);

EXTERN FMTX_API_EXPORT FMTX_API_CALL FMTX_MODE_ENUM fmtxTransmitterGetTuneStatus(double *freq, uint8_t *power, double *rfcap);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxTransmitterSetTuneFreq(double freq);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxTransmitterSetTunePower(uint8_t power, double rfcap);


/* RDS parameters */
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint16_t fmtxRDSGetPI();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t  fmtxRDSSetPI(uint16_t u16Val);

EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSGetPsMixId();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSSetPsMixId(uint8_t u8Val);

EXTERN FMTX_API_EXPORT FMTX_API_CALL uint16_t fmtxRDSGetPsMiscFlags();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t  fmtxRDSSetPsMiscFlags(uint16_t u16Val);

EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSGetPsRepeatCount();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSSetPsRepeatCount(uint8_t u8Val);

EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSGetPsMessageCount();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSSetPsMessageCount(uint8_t u8Val);

EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSGetPsAFStatus(double *freq);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSSetPsAF(double freq, uint8_t enabled);

EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSGetFifoSize();
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSSetFifoSize(uint8_t u8Val);

/* hackish commands for getting PS and RT data from eeprom */
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSGetPsMessageById(uint8_t Id, char *messages);
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSSetPsMessageById(uint8_t Id, char *messages);

EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSGetRtMessage(char *messages); //returns one long string with RT message
EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSSetRtMessage(char *messages);

EXTERN FMTX_API_EXPORT FMTX_API_CALL uint8_t fmtxRDSSendTimeStamp();




#ifdef __cplusplus
}
#endif

#endif // FMTX_H
