/*
 * most processing functions are here
 * $Id$
 * (part of FmStick).
 *
 * (C) 2011 Serge <piratfm@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <cstdlib>
#include <iostream>
#include <QtCore/QString>
#include <QtGui/QApplication>
#include <QMessageBox>
#include <QStatusBar>
#include <QProgressBar>
#include <QIcon>

#include "mainwindow.h"
#include "querier.h"
#include "ui_mainwindow.h"

#include "fmtx.h"
#include "logging.h"

#define REVISION   "$Rev: 268 $"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    int i;
    QLineEdit *curr_msg = NULL;

    liveEditingEnabled=false;
    is_normal = true;
    moving = false;
    acceptClose = false;
    deviceStatus = false;
    toolBarIsVisible = false;
    fmtxCmdSpecial = FMTX_SPECIAL_FRONTEND; ///< At the startup point we want to work with frontend
    fmtxEEPROMInfoClean();

    ui->setupUi(this);

    erroricon = QIcon(":/fmstick/images/Gnome-Network-Wireless-NoTxError-64.png");
    warningicon = QIcon(":/fmstick/images/Gnome-Network-Wireless-NoTxWarning-64.png");
    notxicon = QIcon(":/fmstick/images/Gnome-Network-Wireless-NoTx-64.png");
    txicon = QIcon(":/fmstick/images/fmtransmitter.png");
    nodeviceicon = QIcon(":/fmstick/images/Gnome-Network-Wireless-NoDevice-64.png");

    CurrentVersion = REVISION;
    CurrentVersion.replace(QRegExp("^\\$Rev: (\\d+) \\$$"), "\\1");
    CurrentVersion =  QString(VERSION) + "." + CurrentVersion;
    ui->lineEdit_sw_ver->setText(CurrentVersion);
    ui->checkBox_sw_runAtStartup->setChecked(GetRunAtStartup());
    audioLimiter_prev_div = 0;
    SetAudioCompLimByPreset(4);

    /* actions */
    connect( ui->actionOpen, SIGNAL(triggered()),
             this, SLOT(ReadFrontendParams()) );
    connect( ui->actionSave, SIGNAL(triggered()),
             this, SLOT(SaveFrontendParams()) );

    connect( ui->actionReset_to_Default, SIGNAL(triggered()),
             this, SLOT(resetFmTxSettings()) );
    connect( ui->actionRead_from_Device, SIGNAL(triggered()),
             this, SLOT(GetFrontendParams()) );
    connect( ui->actionWrite_to_Device, SIGNAL(triggered()),
             this, SLOT(SetFrontendParams()) );
    connect( ui->actionRead_from_EEPROM, SIGNAL(triggered()),
             this, SLOT(ShowEEPROMParams()) );
    connect( ui->actionWrite_to_EEPROM, SIGNAL(triggered()),
             this, SLOT(SaveEEPROMParams()) );

    connect( ui->actionSkin_changeMode, SIGNAL(triggered()),
             this, SLOT(toggleWindowMode()) );
    connect( ui->actionToggle_ToolBar, SIGNAL(triggered()),
             this, SLOT(toggleToolBar()));
    connect( ui->actionChange_RadioText, SIGNAL(triggered()),
             this, SLOT(DisplayRadioTextDialog()) );

    connect( ui->actionExit, SIGNAL(triggered()),
             this, SLOT(quit()) );
    connect( ui->actionToggle_transmission, SIGNAL(triggered()),
             this, SLOT(StationButtonClicked()) );
    connect( ui->actionConnect_to_device, SIGNAL(triggered()),
             this, SLOT(SetConnectionState()));
    connect(ui->actionAbout, SIGNAL(triggered()),
            this, SLOT(ShowAboutApp()));

    /* audio compressor/limiter */
    connect( ui->ComboBox_audioPreset, SIGNAL(currentIndexChanged(int)),
             this, SLOT(SetAudioCompLimByPreset(int)) );
    connect( ui->CheckBox_audio_enablecompressor, SIGNAL( stateChanged(int) ),
             this, SLOT( toggleCompressor(int) ) );
    connect( ui->spinBox_audiocompressorThreshold, SIGNAL( valueChanged(int) ),
            this, SLOT( SetAudioCompressorThreshold(int) ) );
    connect( ui->spinBox_audioCompressorGain, SIGNAL(valueChanged(int)),
             this, SLOT(SetAudioCompressorGain(int)));
    connect( ui->comboBox_audioCompressorReleasetime, SIGNAL(currentIndexChanged(int)),
             this, SLOT(SetAudioCompressorReleaseTime(int)));
    connect( ui->doubleSpinBox_audioCompressorAttackTime, SIGNAL(valueChanged(double)),
             this, SLOT(SetAudioCompressorAttackTime(double)));
    connect( ui->checkBox_audioLimiterEnable , SIGNAL( stateChanged(int) ),
             this, SLOT( toggleLimiter(int) ) );
    connect( ui->doubleSpinBox_audio_lim_release_time, SIGNAL(valueChanged(double)),
             this, SLOT(SetAudioLimiterReleaseTime(double)));


    /* audio signal quality measurement */
    connect( ui->checkBox_asq_overmoddetect, SIGNAL( stateChanged(int) ),
             this, SLOT(SetAudioASQFlags()));
    connect( ui->checkBox_asq_detectLow , SIGNAL( stateChanged(int) ),
             this, SLOT( toggleASQLow(int) ) );
    connect( ui->spinBox_asq_detectLowThreshold, SIGNAL(valueChanged(int)),
             this, SLOT(SetAudioASQLevelLow(int)));
    connect( ui->spinBox_asq_detectLowDuration, SIGNAL(valueChanged(int)),
             this, SLOT(SetAudioASQDurationLow(int)));
    connect( ui->checkBox_asq_detectHigh , SIGNAL( stateChanged(int) ),
             this, SLOT( toggleASQHigh(int) ) );
    connect( ui->spinBox_asq_detectHighThreshold, SIGNAL(valueChanged(int)),
             this, SLOT(SetAudioASQLevelHigh(int)));
    connect( ui->spinBox_asq_detectHighDuration, SIGNAL(valueChanged(int)),
             this, SLOT(SetAudioASQDurationHigh(int)));

    /* transmitter configuration */
    //checkBox_transmitter_pilot: spinBox_transmitter_pilot_devi, spinBox_transmitter_pilot_freq, checkBox_transmitter_lmr
    //checkBox_transmitter_rds: spinBox_transmitter_rds_devi
    //checkBox_transmitter_enabled: doubleSpinBox_transmitter_freq spinBox_transmitter_power ==> txcap
    // \==> checkBox_transmitter_antcap: doubleSpinBox_transmitter_antcap_val
    connect( ui->checkBox_transmitter_pilot , SIGNAL( stateChanged(int) ),
             this, SLOT( toggleTransmitPilot(int) ) );
    connect( ui->checkBox_transmitter_rds , SIGNAL( stateChanged(int) ),
             this, SLOT( toggleTransmitRds(int) ) );
    connect( ui->checkBox_transmitter_lmr , SIGNAL( stateChanged(int) ),
             this, SLOT( SetTransmitComponentFlags() ) );

    connect( ui->checkBox_transmitter_enabled , SIGNAL( stateChanged(int) ),
             this, SLOT( toggleTransmitEnabled(int) ) );
    connect( ui->checkBox_transmitter_antcap , SIGNAL( stateChanged(int) ),
             this, SLOT( toggleTransmitAntCap(int) ) );
    connect( ui->comboBox_transmitter_preemph, SIGNAL(currentIndexChanged(int)),
             this, SLOT(SetTransmitPreemphasis(int)));
    connect( ui->spinBox_transmitter_audioDev, SIGNAL(valueChanged(int)),
             this, SLOT(SetTransmitAudioDeviation(int)));

    connect( ui->spinBox_transmitter_pilot_devi, SIGNAL(valueChanged(int)),
             this, SLOT( SetTransmitPilotDeviation(int) ) );
    connect( ui->spinBox_transmitter_pilot_freq, SIGNAL(valueChanged(int)),
             this, SLOT( SetTransmitPilotFrequency(int) ) );
    connect( ui->spinBox_transmitter_rds_devi, SIGNAL(valueChanged(int)),
             this, SLOT( SetTransmitRDSDeviation(int) ) );
    connect( ui->doubleSpinBox_transmitter_freq, SIGNAL(valueChanged(double)),
             this, SLOT( SetTransmitFrequency(double) ) );
    connect( ui->spinBox_transmitter_power, SIGNAL(valueChanged(int)),
             this, SLOT( SetTransmitPower() ) );
    connect( ui->doubleSpinBox_transmitter_antcap_val, SIGNAL(valueChanged(double)),
             this, SLOT( SetTransmitPower() ) );

    /* RDS Configuration */
    //checkBox_rds_useaf: doubleSpinBox_rds_af
    //checkBox_rds_fifo: spinBox_rds_fifo_sz
    // \==> checkBox_rds_time: spinBox_rds_time_int
    connect( ui->lineEdit_rds_progid, SIGNAL( editingFinished()),
             this, SLOT( SetRDSPI() ));
    connect( ui->checkBox_rds_useaf , SIGNAL( stateChanged(int) ),
             this, SLOT( toggleRDSaf(int) ) );
    connect( ui->doubleSpinBox_rds_af, SIGNAL(valueChanged(double)),
             this, SLOT( SetRDSAF(double) ));
    connect( ui->checkBox_rds_fifo , SIGNAL( stateChanged(int) ),
             this, SLOT( toggleRDSfifo(int) ) );
    connect( ui->spinBox_rds_fifo_sz , SIGNAL(valueChanged(int)),
             this, SLOT( SetRDSFIFO(int) ) );
    connect( ui->spinBox_rds_ps_msgs , SIGNAL( valueChanged(int) ),
             this, SLOT( toggleRDSPSNum(int) ) );
    connect( ui->spinBox_rds_ps_repeats , SIGNAL( valueChanged(int) ),
             this, SLOT( SetRDSPSRepeats(int) ) );
    connect( ui->comboBox_rds_ps_percentage, SIGNAL(currentIndexChanged(int)),
             this, SLOT(SetRDSPSPercentage(int)));

    /* this boths PS msg editors must not re-block each other */
    connect( ui->lineEdit_station_id, SIGNAL(textChanged(QString)), this, SLOT( toggleRDSPSMsg(QString)) );
    for(i=0; i<12; i++){
        curr_msg = GetRDSLineEditById(i);
        //if(curr_msg) connect( curr_msg, SIGNAL(editingFinished()), this, SLOT(toggleStationId()) );
        //if(curr_msg) connect( curr_msg, SIGNAL(textEdited(QString)), this, SLOT(toggleStationId()) );
        if(curr_msg) connect( curr_msg, SIGNAL(textChanged(QString)), this, SLOT(toggleStationId()) );
    }

    connect( ui->plainTextEdit_rds_rt, SIGNAL(textChanged()),
             this, SLOT(toggleRDSRT()));

    /* misc flags */
    connect( ui->checkBox_rds_art_head, SIGNAL(stateChanged(int)),
             this, SLOT(SetRDSPsMiscFlags()) );
    connect( ui->checkBox_rds_compr_code, SIGNAL(stateChanged(int)),
             this, SLOT(SetRDSPsMiscFlags()) );
    connect( ui->checkBox_rds_dynPT, SIGNAL(stateChanged(int)),
             this, SLOT(SetRDSPsMiscFlags()) );
    connect( ui->checkBox_rds_forceb, SIGNAL(stateChanged(int)),
             this, SLOT(SetRDSPsMiscFlags()) );
    connect( ui->checkBox_rds_is_stereo, SIGNAL(stateChanged(int)),
             this, SLOT(SetRDSPsMiscFlags()) );
    connect( ui->checkBox_rds_ta, SIGNAL(stateChanged(int)),
             this, SLOT(SetRDSPsMiscFlags()) );
    connect( ui->checkBox_rds_ta_tp, SIGNAL(stateChanged(int)),
             this, SLOT(SetRDSPsMiscFlags()) );
    connect( ui->comboBox_rds_pty, SIGNAL(currentIndexChanged(int)),
             this, SLOT(SetRDSPsMiscFlags()) );


    /* StationId -> PS msg editor */
    //connect( ui->lineEdit_station_id, SIGNAL(textEdited(QString)),
    //         this, SLOT( toggleRDSPSMsg(QString)) );

    /* BigButton actions */
    connect( ui->commandLinkButton_station, SIGNAL(clicked()),
             this, SLOT(StationButtonClicked()) );
    connect( ui->commandLinkButton_station, SIGNAL(customContextMenuRequested(const QPoint &)),
             this, SLOT(StationButtonContextMenu(const QPoint &)) );

    /* hardware section */
    connect(ui->pushButton_fe_powerdown, SIGNAL(clicked()),
            this, SLOT(SetFrontendPowerDown()));
    connect(ui->pushButton_fe_powerup, SIGNAL(clicked()),
            this, SLOT(SetFrontendPowerUp()));
    connect(ui->pushButton_fe_reset, SIGNAL(clicked()),
            this, SLOT(SetFrontendReset()));
    connect( ui->pushButton_open_firmware, SIGNAL(clicked()),
             this, SLOT(UpdateFirmware()));
    connect( ui->checkBox_hw_run_txAtPowerUp , SIGNAL( toggled(bool) ),
             this, SLOT( SetRunEepromCfg(bool) ) );

    /* software configuration */
    connect( ui->checkBox_rds_time , SIGNAL( stateChanged(int) ),
             this, SLOT( toggleRDStime(int) ) );
    connect( ui->spinBox_rds_time_int , SIGNAL( valueChanged(int) ),
             this, SLOT( SetRDSTime(int) ) );
    connect( ui->checkBox_sw_runAtStartup, SIGNAL(stateChanged(int)),
             this, SLOT(SetRunAtStartup(int)) );
    connect( ui->checkBox_sw_live_edit, SIGNAL(toggled(bool)),
             this, SLOT(toggleLiveEditing(bool)));
    connect( ui->pushButton_sw_visitWebsite, SIGNAL(clicked()),
             this, SLOT(goToHomePage()));

#ifdef QT_NO_DEBUG
    loglevel=LOG_INFO;
#else
    loglevel=LOG_DUMP;
#endif

    querier = new Querier();
    /* actions from querier */
    connect( querier, SIGNAL(setASQOvermod(bool)), ui->label_audio_overmodulated, SLOT(setEnabled(bool)) );
    connect( querier, SIGNAL(setASQHighLevel(bool)),
             ui->label_audio_asq_triggered_high, SLOT(setEnabled(bool)) );
    connect( querier, SIGNAL(setASQLowLevel(bool)), ui->label_audio_asq_triggered_low, SLOT(setEnabled(bool)) );
    connect( querier, SIGNAL(setASQLevel(int)), this, SLOT(setAudioMeasuredLvl(int)) );
    connect( querier, SIGNAL(setTransmitterAntCap(double)),
             this, SLOT(setAntCapNoEvent(double)) );
    connect( querier, SIGNAL(setTransmitterTuned(int)), this, SLOT( SetTransmitIcon(int) ) );
    connect( querier, SIGNAL(setFrontendFound(bool)), this, SLOT(RedrawEverything(bool)) );

    //connect( ui->actionSelect_Language, SIGNAL(triggered(bool)), this, SLOT(showLanguageMenu(bool)) );
    /* read saved state */
    readSettings(&settings, false);

    switchLanguage(NULL);

    createLanguageMenu();
    ui->actionSelect_Language->setMenu(languageMenu);

    setupTrayIcon();

    SetWindowMode(!ui->checkBox_mini_mode->isChecked());
    SetAudioCompLimPresetByParams();

    /* actions for querier */
    if(ui->checkBox_rds_time->isChecked()) {
        querier->rdsInterval = ui->spinBox_rds_time_int->value() * 1000;
    }

    /* try to connect */
    toggleConnection(false, false);
    if(ui->checkBox_sw_autoconnect->isChecked())
        querier->timer.start();

    qDebug() << __FUNCTION__;
}

void MainWindow::retranslateUi()
{
    ui->retranslateUi(this);
    //menu->setTitle(tr("File"));
    //fileAction->setText(tr("First action"));
}

void MainWindow::setAudioMeasuredLvl(int iVal)
{
    ui->label_asq_level->setText(QString("%1").arg(iVal));
    if(iVal < -70)
        iVal = -70;
    if(iVal > 0)
        iVal = 0;
    ui->progressBar_audio_asq_measured->setValue(iVal);
}

void MainWindow::setAntCapNoEvent(double dVal)
{
    ui->doubleSpinBox_transmitter_antcap_val->blockSignals(true);
    ui->doubleSpinBox_transmitter_antcap_val->setValue(dVal);
    ui->doubleSpinBox_transmitter_antcap_val->blockSignals(false);
}

MainWindow::~MainWindow()
{
    delete ui;
//    if(querier->isRunning())
//        querier->terminate();
    delete querier;

    trayIcon->setVisible(false);
    trayIcon->hide();
    delete trayIcon;
}


void MainWindow::toggleLiveEditing(bool bVal)
{
    if(deviceStatus)
        liveEditingEnabled = bVal;
}

void MainWindow::SetConnectionState()
{
    qDebug() << __FUNCTION__;
    /* force querier to disconnect */
    liveEditingEnabled = false;
    querier->device_lost = true;
    querier->still_not_exists = false;
    toggleConnection(!deviceStatus, true);

    if(deviceStatus)
        querier->timer.setInterval(0);
}

void MainWindow::toggleConnection(bool bVal, bool toggleQuerier)
{
    qDebug() << __FUNCTION__ << ": " << bVal;
    /* actions */
    ui->actionRead_from_Device->setEnabled(bVal);
    ui->actionRead_from_EEPROM->setEnabled(bVal);
    ui->actionWrite_to_Device->setEnabled(bVal);
    ui->actionWrite_to_EEPROM->setEnabled(bVal);
    ui->actionToggle_transmission->setEnabled(bVal);
    /* hardware buttons */
    ui->checkBox_hw_run_txAtPowerUp->setEnabled(bVal);
    ui->pushButton_fe_powerdown->setEnabled(bVal);
    ui->pushButton_fe_powerup->setEnabled(bVal);
    ui->pushButton_fe_reset->setEnabled(bVal);
    ui->pushButton_open_firmware->setEnabled(bVal);

    if(bVal) {
        if(toggleQuerier)
            querier->timer.start();
        SetTransmitIcon(ICON_WARN);
        ui->actionConnect_to_device->setText("Disconnect?");
        ui->actionConnect_to_device->setIcon(QIcon(":/fmstick/images/connect_32.png"));
    } else {
        if(toggleQuerier) {
            querier->timer.stop();
        }
        SetTransmitIcon(ICON_NODEV);
        ui->actionConnect_to_device->setIcon(QIcon(":/fmstick/images/disconnect_32.png"));
        ui->actionConnect_to_device->setText("Connect?");
    }
    deviceStatus = bVal;
}


void MainWindow::SetFrontendPowerDown() {
    qDebug() << __FUNCTION__;
    if(fmtxIoAppFeDown() != FMTX_MODE_POWER_DOWN) {
        qDebug() << "error while powering down";
    }
    SetTransmitIcon(ICON_WARN);
    querier->is_ready=false;
    liveEditingEnabled=false;
    ui->actionRead_from_Device->setEnabled(false);
    ui->actionWrite_to_Device->setEnabled(false);
    fmtxCmdStatus=FMTX_MODE_NONE;
}

int MainWindow::SetFrontendPowerUp() {
    int ret;
    qDebug() << __FUNCTION__;
    querier->HidAccessMutex.lock();
    ret = fmtxIoAppFeUp();
    querier->HidAccessMutex.unlock();
    switch(ret){
    case FMTX_MODE_POWER_UP:
        qDebug() << "Frontend powered up!";
        SetTransmitIcon(ICON_NOTX);
        fmtxCmdStatus=FMTX_MODE_OK;
        break;
    case FMTX_MODE_TRANSMITTING:
        qDebug() << "Frontend already powered up!";
        SetTransmitIcon(ICON_TX);
        fmtxCmdStatus=FMTX_MODE_OK;
        break;
    default:
        qDebug() << "Error while powering up";
        fmtxCmdStatus=FMTX_MODE_NONE;
        break;
    }

    if(ret == FMTX_MODE_POWER_UP || ret == FMTX_MODE_TRANSMITTING) {
        querier->is_ready=true;
        liveEditingEnabled=ui->checkBox_sw_live_edit->isChecked();
        ui->actionRead_from_Device->setEnabled(true);
        ui->actionWrite_to_Device->setEnabled(true);
    }

    return ret;
}

void MainWindow::SetFrontendReset() {
    qDebug() << __FUNCTION__;
    if(fmtxIoAppFeReset() != FMTX_MODE_POWER_DOWN)
        qDebug() << "Error while resetting frontend";
    querier->is_ready=false;
    liveEditingEnabled=false;
    SetTransmitIcon(ICON_WARN);
    ui->actionRead_from_Device->setEnabled(false);
    ui->actionWrite_to_Device->setEnabled(false);
    fmtxCmdStatus=FMTX_MODE_NONE;
}

void MainWindow::StationButtonClicked()
{
    qDebug() << __FUNCTION__;
    bool is_checked = ui->checkBox_transmitter_enabled->isChecked();
    ui->checkBox_transmitter_enabled->setChecked(!is_checked);
}


int MainWindow::GetFrontendParams()
{
    qDebug() << __FUNCTION__;
    liveEditingEnabled=false;

    DisplayAudioEditor();
    if(fmtxCmdStatus!=FMTX_MODE_OK)
        goto frontend_params_err;

    DisplayTransmitterEditor();
    if(fmtxCmdStatus!=FMTX_MODE_OK)
        goto frontend_params_err;

    DisplayRDSEditor();
    if(fmtxCmdStatus!=FMTX_MODE_OK)
        goto frontend_params_err;
    liveEditingEnabled=ui->checkBox_sw_live_edit->isChecked();
    /* run querier, so we can read interrupts and send rds timestamps */
    return fmtxCmdStatus;

frontend_params_err:
    liveEditingEnabled=false;
    qDebug() << "Can't get frontend parameters";
    SetTransmitIcon(ICON_ERR);
    return fmtxCmdStatus;
}

int MainWindow::SetFrontendParams()
{
    qDebug() << __FUNCTION__;
    liveEditingEnabled=true;

    ConfigureAudioEditor();
    if(fmtxCmdStatus!=FMTX_MODE_OK)
        goto frontend_params_err;

    ConfigureTransmitterEditor();
    if(fmtxCmdStatus!=FMTX_MODE_OK)
        goto frontend_params_err;

    ConfigureRDSEditor();
    if(fmtxCmdStatus!=FMTX_MODE_OK)
        goto frontend_params_err;

    if(!SetDigitalAudioHack())
        goto frontend_params_err;

    /* run querier, so we can read interrupts and send rds timestamps */
    qDebug() << "Frontend parameters setted correctly.";
    liveEditingEnabled=ui->checkBox_sw_live_edit->isChecked();
    return fmtxCmdStatus;

frontend_params_err:
    liveEditingEnabled=false;
    qDebug() << "Can't set frontend parameters";
    SetTransmitIcon(ICON_ERR);
    return fmtxCmdStatus;
}

void MainWindow::RedrawEverything(bool bFound)
{
    int ret, transmitting;
    char cpuid[32];
    char rev[32];
    QStatusBar *status = statusBar();
    qDebug() << __FUNCTION__;
    fmtxEEPROMInfoClean();

    liveEditingEnabled=false;
    querier->is_ready=false;
    deviceStatus=bFound;

    toggleConnection(bFound, false);
    if(!bFound) {
        querier->is_ready=ui->checkBox_sw_autoconnect->isChecked();
        if(!ui->checkBox_sw_autoconnect->isChecked())
            querier->timer.stop();
        qDebug() << "frontend flagged as" << bFound << " returning.";
        if(!is_normal || !this->isVisible())
            trayIcon->showMessage(tr("FmStick disconnected"),
                        tr("Device disconnected!"),
                        QSystemTrayIcon::Information, 5 * 1000);
        return;
    }

    status->showMessage(tr("(Re)Reading frontend info..."), 10000);
    status->show();

    cpuid[0] = '\0';
    rev[0] = '\0';
    querier->HidAccessMutex.lock();
    ret = fmtxIoAppIdString(cpuid, rev);
    querier->HidAccessMutex.unlock();
    if(ret!=FMTX_MODE_OK) {
        qDebug() << "can't get id string";
        goto redraw_err;
    }
    qDebug() << "got CPU ID: "<<QString(cpuid)<<" rev.:"<<QString(rev);
    ui->lineEdit_hw_id_cpuid->setText(QString(cpuid));
    ui->lineEdit_hw_id_cpufwid->setText(QString(rev));


    status->showMessage(tr("Reading EEPROM..."), 10000);
    status->show();
    qDebug() << "Reading EEPROM...";
    querier->HidAccessMutex.lock();
    fmtxCmdSpecial = FMTX_SPECIAL_EEPROM;   ///< now work with eeprom instead of frontend
    ret=fmtxEEPROMReadConfig();
    fmtxCmdSpecial = FMTX_SPECIAL_FRONTEND;
    querier->HidAccessMutex.unlock();

    ui->checkBox_hw_run_txAtPowerUp->blockSignals(true);
    ui->checkBox_hw_run_txAtPowerUp->setChecked(fmtxEEPROMGetStartupFlag());
    ui->checkBox_hw_run_txAtPowerUp->blockSignals(false);

    if(ret==FMTX_MODE_POWER_DOWN) {
        qDebug() << "EEPROM is empty yet!";
    } else if(ret!=FMTX_MODE_OK) {
        qDebug() << "Can't read EEPROM";
        goto redraw_err;
    }
    qDebug() << "EEPROM Readed..";


    transmitting = 0;
    querier->HidAccessMutex.lock();
    ret = fmtxIoAppFeGetMode(&transmitting);
    querier->HidAccessMutex.unlock();
    qDebug() << "Frontend mode: " << ret << "transmitting: " << transmitting;
    if(ret == FMTX_MODE_POWER_DOWN){
        qDebug() << "Frontend powered down!";
        ret = SetFrontendPowerUp();
    }

    if(ret!=FMTX_MODE_POWER_UP && ret!=FMTX_MODE_TRANSMITTING) {
        qDebug() << "Frontend not found..";
        SetTransmitIcon(ICON_WARN);
        goto redraw_err;
    }

    if(transmitting) {
        qDebug() << "Already transmitting!";
        SetTransmitIcon(ICON_TX);
    }

    /* needed to detect if hardware supports rds */
    querier->HidAccessMutex.lock();
    DisplayFirmwareInfo();
    querier->HidAccessMutex.unlock();
    if(fmtxCmdStatus!=FMTX_MODE_OK)
        goto redraw_err;

    if(ui->checkBox_rewriteCfg->isChecked()) {
        qDebug() << "Forcing configuration from software, instead of hardware!";
        status->showMessage("Writing running config...", 10000);
        status->show();
        if(SetFrontendParams()!=FMTX_MODE_OK)
            goto redraw_err;
        //hack for digital audio
        if(! SetDigitalAudioHack())
                goto redraw_err;
    }

    status->showMessage(tr("Reading running config..."), 10000);
    status->show();
    querier->HidAccessMutex.lock();
    ret=GetFrontendParams();
    querier->HidAccessMutex.unlock();

    if(ret!=FMTX_MODE_OK)
        goto redraw_err;

    liveEditingEnabled=ui->checkBox_sw_live_edit->isChecked();
    statusBar()->hide();
    this->adjustSize();
    this->resize(1,1);
    this->updateGeometry();
    querier->is_ready=true;

    qDebug() << "Redraw everything done! transmitting: " << transmitting;
    if(!is_normal || !this->isVisible())
        trayIcon->showMessage(tr("FmStick connected"),
                    tr("Device found and connected!%1")
                              .arg( transmitting ? tr("\nAnd transmitter is powered on!") : ""),
                    QSystemTrayIcon::Information, 5 * 1000);
    return;

redraw_err:
    if(!is_normal || !this->isVisible())
        trayIcon->showMessage(tr("FmStick not connected"), tr("Can't connect to frontend!"),
                          QSystemTrayIcon::Critical, 5 * 1000);
    status->showMessage(tr("Can't connect to frontend!"), 10000);
    status->show();
    qDebug() << "Error while redraw everything";
    querier->is_ready=true;
    SetTransmitIcon(ICON_ERR);
}

QLineEdit *MainWindow::GetRDSLineEditById(int id)
{
    switch(id){
    case 0:
        return ui->lineEdit_rds_ps_1;
        break;
    case 1:
        return ui->lineEdit_rds_ps_2;
        break;
    case 2:
        return ui->lineEdit_rds_ps_3;
        break;
    case 3:
        return ui->lineEdit_rds_ps_4;
        break;
    case 4:
        return ui->lineEdit_rds_ps_5;
        break;
    case 5:
        return ui->lineEdit_rds_ps_6;
        break;
    case 6:
        return ui->lineEdit_rds_ps_7;
        break;
    case 7:
        return ui->lineEdit_rds_ps_8;
        break;
    case 8:
        return ui->lineEdit_rds_ps_9;
        break;
    case 9:
        return ui->lineEdit_rds_ps_10;
        break;
    case 10:
        return ui->lineEdit_rds_ps_11;
        break;
    case 11:
        return ui->lineEdit_rds_ps_12;
        break;
    }
    return NULL;
}

void MainWindow::DelayedSendPS()
{
    int i;
    for(i=0; i<ui->spinBox_rds_ps_msgs->value(); i++){
        if(SetRDSPSMessage(i) != FMTX_MODE_OK)
            return;
    }
}


void MainWindow::toggleStationId()
{
    qDebug() << __FUNCTION__;
    int i;
    QString NewId, txt;
    QLineEdit *curr_msg = NULL;
    for(i=0; i<ui->spinBox_rds_ps_msgs->value(); i++){
        curr_msg = GetRDSLineEditById(i);
        if(curr_msg) {
            if(i+1 == ui->spinBox_rds_ps_msgs->value()) {
                txt.clear();
                txt.append(curr_msg->text());
            } else {
                txt.fill(' ', 8);
                txt.replace(0, curr_msg->text().length(), curr_msg->text());
            }
            NewId.append(txt);
        }
    }

    while(!NewId.isEmpty() && NewId.endsWith(' '))
        NewId.chop(1);

    qDebug() << "PS Msg: " << NewId;
    ui->lineEdit_station_id->blockSignals(true);
    ui->lineEdit_station_id->setText(NewId);
    ui->lineEdit_station_id->blockSignals(false);
    DelayedSendPS();
}

void MainWindow::toggleRDSPSMsg(QString msg)
{
    qDebug() << __FUNCTION__;
    int i, len = msg.length() / 8;
    QLineEdit *curr_msg;
    if (msg.length() % 8)
        len++;
    for(i=0; i<len; i++){
        QString src = msg.mid(8*i, 8);
        curr_msg = GetRDSLineEditById(i);
        if(curr_msg) {
            curr_msg->blockSignals(true);
            curr_msg->setText(src);
            curr_msg->blockSignals(false);
        }
    }
    if(!msg.length()) ui->lineEdit_rds_ps_1->clear();
    ui->spinBox_rds_ps_msgs->setValue(len ? len : 1);
    DelayedSendPS();
}

void MainWindow::DisplayRDSEditor(void)
{
    qDebug() << __FUNCTION__;
    uint16_t rdsFlags = 0;
    double rdsAFFreq = 0;
    uint8_t fifo_sz;
    int ps_msgs, i;
    char psmsgbuff[9];
    char rtmsgbuff[65];
    QString msgPart = "";

    qDebug() << "Reading RDS info...";

    //lineEdit_rds_progid
    ui->lineEdit_rds_progid->setText(QString("%1").arg(fmtxRDSGetPI(), 4, 16).toUpper());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_rds_err;

    rdsFlags = fmtxRDSGetPsMiscFlags();
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_rds_err;

    ui->checkBox_rds_dynPT->setChecked(rdsFlags & TX_RDS_PS_MISC_RDSD3_MASK);
    ui->checkBox_rds_compr_code->setChecked(rdsFlags & TX_RDS_PS_MISC_RDSD2_MASK);
    ui->checkBox_rds_art_head->setChecked(rdsFlags & TX_RDS_PS_MISC_RDSD1_MASK);
    ui->checkBox_rds_is_stereo->setChecked(rdsFlags & TX_RDS_PS_MISC_RDSD0_MASK);
    ui->checkBox_rds_forceb->setChecked(rdsFlags & TX_RDS_PS_MISC_FORCEB_MASK);
    ui->radioButton_rds_music->setChecked(rdsFlags & TX_RDS_PS_MISC_RDSMS_MASK);
    ui->radioButton_rds_speech->setChecked(!(rdsFlags & TX_RDS_PS_MISC_RDSMS_MASK));
    ui->comboBox_rds_pty->setCurrentIndex((rdsFlags & TX_RDS_PS_MISC_RDSPTY_MASK) >> TX_RDS_PS_MISC_RDSPTY_SHFT);

    //checkBox_rds_useaf
    ui->checkBox_rds_useaf->setChecked(fmtxRDSGetPsAFStatus(&rdsAFFreq));
    toggleRDSaf(ui->checkBox_rds_useaf->isChecked());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_rds_err;

    //doubleSpinBox_rds_af
    ui->doubleSpinBox_rds_af->setValue(rdsAFFreq);

    fifo_sz = fmtxRDSGetFifoSize();
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_rds_err;

    ui->checkBox_rds_fifo->setChecked(fifo_sz > 0);
    toggleRDSfifo(fifo_sz > 0);
    ui->spinBox_rds_fifo_sz->setValue(fifo_sz);

    //it's software dependant!
    //checkBox_rds_time
    //spinBox_rds_time_int

    //comboBox_rds_ps_percentage
    ui->comboBox_rds_ps_percentage->setCurrentIndex(fmtxRDSGetPsMixId());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_rds_err;

    //spinBox_rds_ps_repeats
    ui->spinBox_rds_ps_repeats->setValue(fmtxRDSGetPsRepeatCount());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_rds_err;

    ps_msgs = fmtxRDSGetPsMessageCount();
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_rds_err;

    /* No Chance to read this from running frontend */
    if(!ui->checkBox_rewriteCfg->isChecked() || fmtxCmdSpecial == FMTX_SPECIAL_EEPROM) {
        for(i=0; i<ps_msgs; i++)
        {
            fmtxRDSGetPsMessageById(i, psmsgbuff);
            if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_rds_err;

            msgPart = QString(psmsgbuff);
            while(!msgPart.isEmpty() && msgPart.endsWith(' '))
                msgPart.chop(1);

            qDebug() << "PS Msg " << i << ": " << msgPart;
            GetRDSLineEditById(i)->setText(msgPart);
        }
    }

    fmtxRDSGetRtMessage(rtmsgbuff);
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_rds_err;
    rtmsgbuff[64] = 0;
    ui->plainTextEdit_rds_rt->setPlainText(QString(rtmsgbuff));

    ui->checkBox_rds_ta->setChecked(rdsFlags & TX_RDS_PS_MISC_RDSTA_MASK);
    ui->checkBox_rds_ta_tp->setChecked(rdsFlags & TX_RDS_PS_MISC_RDSTP_MASK);

    ui->spinBox_rds_ps_msgs->setValue(ps_msgs);

    qDebug() << "RDS info readed";
    return;

display_rds_err:
    qDebug() << "Error while reading RDS info";
}

void MainWindow::ConfigureRDSEditor(void)
{
    int i;
    QString str;
    qDebug() << "Writing RDS info...";

    SetRDSPsMiscFlags();
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_rds_err;

    SetRDSPI();
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_rds_err;

    SetRDSAF(ui->doubleSpinBox_rds_af->value());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_rds_err;

    if(ui->checkBox_rds_fifo->isChecked())
        SetRDSFIFO(ui->spinBox_rds_fifo_sz->value());
    else
        SetRDSFIFO(0);
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_rds_err;

    SetRDSPSNum(ui->spinBox_rds_ps_msgs->value());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_rds_err;

    for(i=0; i<ui->spinBox_rds_ps_msgs->value(); i++)
        if(SetRDSPSMessage(i) != FMTX_MODE_OK)
            goto configure_rds_err;

    SetRDSPSRepeats(ui->spinBox_rds_ps_repeats->value());
    SetRDSPSPercentage(ui->comboBox_rds_ps_percentage->currentIndex());

    str = ui->plainTextEdit_rds_rt->toPlainText();
    SetRDSRT(&str);

    qDebug() << "RDS info sended";
    return;

configure_rds_err:
    qDebug() << "Error while writing RDS info";
}

void MainWindow::DisplayTransmitterEditor(void)
{
    uint16_t transmitterFlags = 0;
    uint8_t fmtxStatus;
    uint8_t power=0;
    double freq=0.0;
    double rfcap=0;

    qDebug() << "Reading Transmitter info...";

    transmitterFlags=fmtxTransmitterGetComponentFlags();
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_transmitter_err;

    ui->comboBox_transmitter_preemph->setCurrentIndex(fmtxTransmitterGetPreemphasisId());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_transmitter_err;

    ui->spinBox_transmitter_audioDev->setValue(fmtxTransmitterGetAudioDeviation());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_transmitter_err;

    ui->checkBox_transmitter_pilot->setChecked(transmitterFlags & TX_COMPONENT_ENABLE_PILOT_MASK);
    toggleTransmitPilot(transmitterFlags & TX_COMPONENT_ENABLE_PILOT_MASK);

    ui->spinBox_transmitter_pilot_devi->setValue(fmtxTransmitterGetPilotDeviation());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_transmitter_err;

    ui->spinBox_transmitter_pilot_freq->setValue(fmtxTransmitterGetPilotFrequency());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_transmitter_err;

    ui->checkBox_transmitter_lmr->setChecked(transmitterFlags & TX_COMPONENT_ENABLE_LMR_MASK);

    ui->checkBox_transmitter_rds->setChecked(transmitterFlags & TX_COMPONENT_ENABLE_RDS_MASK);
    toggleTransmitRds(transmitterFlags & TX_COMPONENT_ENABLE_RDS_MASK);

    //spinBox_transmitter_rds_devi
    ui->spinBox_transmitter_rds_devi->setValue(fmtxTransmitterGetRDSDeviation());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_transmitter_err;

    fmtxStatus = fmtxTransmitterGetTuneStatus(&freq, &power, &rfcap);
    qDebug() << "Transmitter configuration("<< fmtxCmdStatus <<"): freq:" << freq << ", power:" << power << ", rfcap:" << rfcap;
    if(fmtxCmdStatus!=FMTX_MODE_OK && fmtxCmdStatus!=FMTX_MODE_TRANSMITTING) goto display_transmitter_err;

    ui->checkBox_transmitter_enabled->setChecked(power>0);
    toggleTransmitEnabled(power>0);
    SetTransmitIcon(power>0 ? ICON_TX : ICON_NOTX);

    ui->doubleSpinBox_transmitter_freq->setValue(freq);

    ui->spinBox_transmitter_power->setValue(power);

    //TODO: we dont know who setted this value, user or ic
    //ui->checkBox_transmitter_antcap->setChecked(false);
    //toggleTransmitAntCap(0);
    //doubleSpinBox_transmitter_antcap_val
    ui->doubleSpinBox_transmitter_antcap_val->setValue(rfcap);

    qDebug() << "Transmitter info readed";
    return;

display_transmitter_err:
    qDebug() << "Error while reading transmitter info";
}

void MainWindow::ConfigureTransmitterEditor(void)
{
    int ret;

    qDebug() << "Setting Transmitter info...";

    SetTransmitPreemphasis(ui->comboBox_transmitter_preemph->currentIndex());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_transmitter_err;

    SetTransmitAudioDeviation(ui->spinBox_transmitter_audioDev->value());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_transmitter_err;

    SetTransmitPilotDeviation(ui->spinBox_transmitter_pilot_devi->value());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_transmitter_err;

    SetTransmitPilotFrequency(ui->spinBox_transmitter_pilot_freq->value());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_transmitter_err;

    SetTransmitRDSDeviation(ui->spinBox_transmitter_rds_devi->value());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_transmitter_err;

    SetTransmitFrequency(ui->doubleSpinBox_transmitter_freq->value());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_transmitter_err;

    querier->HidAccessMutex.lock();
    if(ui->checkBox_transmitter_enabled->isChecked()) {
        if(ui->checkBox_transmitter_antcap->isChecked())
            ret=fmtxTransmitterSetTunePower(ui->spinBox_transmitter_power->value(), ui->doubleSpinBox_transmitter_antcap_val->value());
        else
            ret=fmtxTransmitterSetTunePower(ui->spinBox_transmitter_power->value(), 0.0);
        SetTransmitIcon(ICON_TX);
    } else {
        ret=fmtxTransmitterSetTunePower(0, 0.0);
        SetTransmitIcon(ICON_NOTX);
    }
    querier->HidAccessMutex.unlock();
    if(ret != FMTX_MODE_OK) {
        qDebug() << "Can't enable transmission";
        SetTransmitIcon(ICON_WARN);
        goto configure_transmitter_err;
    }

    SetTransmitComponentFlags();
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_transmitter_err;

    qDebug() << "Transmitter info sended";
    return;

configure_transmitter_err:
    qDebug() << "Error while sending transmitter info";
}

void MainWindow::DisplayAudioEditor(void)
{
    uint8_t audioFlags;
    int16_t audioAsqFlags = 0;

    qDebug() << "Reading Audio info...";
    audioFlags=fmtxAudioGetAcompFlags();
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_audio_err;

    ui->CheckBox_audio_enablecompressor->setChecked(audioFlags & TX_ACOMP_ENABLE_ACEN_MASK);
    toggleCompressor(audioFlags & TX_ACOMP_ENABLE_ACEN_MASK);

    ui->comboBox_audioCompressorReleasetime->setCurrentIndex(fmtxAudioGetAcompReleaseTimeId());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_audio_err;

    ui->doubleSpinBox_audioCompressorAttackTime->setValue(fmtxAudioGetAcompAttackTime());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_audio_err;
    //(0.5 * (tmpprop+1)

    ui->spinBox_audioCompressorGain->setValue(fmtxAudioGetAcompGain());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_audio_err;

    ui->spinBox_audiocompressorThreshold->setValue(fmtxAudioGetAcompThreshold());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_audio_err;

    ui->checkBox_audioLimiterEnable->setChecked(audioFlags & TX_ACOMP_ENABLE_LIMITEN_MASK);
    toggleLimiter(audioFlags & TX_ACOMP_ENABLE_LIMITEN_MASK);

    ui->doubleSpinBox_audio_lim_release_time->setValue(fmtxAudioGetLimiterReleaseTime());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_audio_err;
    //512/float
    //change audio preset combobox
    SetAudioCompLimPresetByParams();

    audioAsqFlags = fmtxAudioGetAsqIntFlags();
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_audio_err;

    ui->checkBox_asq_overmoddetect->setChecked(audioAsqFlags & TX_ASQ_INT_SELECT_OVERMODIEN_MASK);

    ui->checkBox_asq_detectHigh->setChecked(audioAsqFlags & TX_ASQ_INT_SELECT_IALHIEN_MASK);
    toggleASQHigh(audioAsqFlags & TX_ASQ_INT_SELECT_IALHIEN_MASK);

    ui->spinBox_asq_detectHighDuration->setValue(fmtxAudioGetAsqDurationHigh());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_audio_err;

    ui->spinBox_asq_detectHighThreshold->setValue(fmtxAudioGetAsqLevelHigh());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_audio_err;

    ui->checkBox_asq_detectLow->setChecked(audioAsqFlags & TX_ASQ_INT_SELECT_IALLIEN_MASK);
    toggleASQLow(audioAsqFlags & TX_ASQ_INT_SELECT_IALLIEN_MASK);

    ui->spinBox_asq_detectLowDuration->setValue(fmtxAudioGetAsqDurationLow());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_audio_err;

    ui->spinBox_asq_detectLowThreshold->setValue(fmtxAudioGetAsqLevelLow());
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto display_audio_err;

    qDebug() << "Audio info readed";
    return;

display_audio_err:
    qDebug() << "Error while reading audio info";
}

void MainWindow::ConfigureAudioEditor(void)
{

    qDebug() << "Sending Audio info...";
    SetAudioCompLimFlags();

    if(ui->CheckBox_audio_enablecompressor->isChecked()) {
        SetAudioCompressorThreshold(ui->spinBox_audiocompressorThreshold->value());
        if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_audio_err;

        SetAudioCompressorGain(ui->spinBox_audioCompressorGain->value());
        if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_audio_err;

        SetAudioCompressorReleaseTime(ui->comboBox_audioCompressorReleasetime->currentIndex());
        if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_audio_err;

        SetAudioCompressorAttackTime(ui->doubleSpinBox_audioCompressorAttackTime->value());
        if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_audio_err;
    }

    if(ui->CheckBox_audio_enablecompressor->isChecked()) {
        SetAudioLimiterReleaseTime(ui->doubleSpinBox_audio_lim_release_time->value());
        if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_audio_err;
    }

    SetAudioASQFlags();
    if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_audio_err;

    if(ui->checkBox_asq_detectLow->isChecked()) {
        SetAudioASQLevelLow(ui->spinBox_asq_detectLowThreshold->value());
        if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_audio_err;

        SetAudioASQDurationLow(ui->spinBox_asq_detectLowDuration->value());
        if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_audio_err;
    }

    if(ui->checkBox_asq_detectHigh->isChecked()) {
        SetAudioASQLevelHigh(ui->spinBox_asq_detectHighThreshold->value());
        if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_audio_err;

        SetAudioASQDurationHigh(ui->spinBox_asq_detectHighDuration->value());
        if(fmtxCmdStatus!=FMTX_MODE_OK) goto configure_audio_err;
    }

    qDebug() << "Audio info sended";
    return;

configure_audio_err:
    qDebug() << "Error while sending audio info";
}

void MainWindow::DisplayFirmwareInfo(void)
{
    uint8_t buff[16];
    QString digistr;
    QChar fw_major, fw_minor;

    qDebug() << "Reading Firmware info...";

    if((fmtxCmdStatus = fmtxIoAppCommand(GET_REV, buff, 0)) !=FMTX_MODE_OK) goto display_firmware_err;

    digistr = QString::number(buff[1]).rightJustified(2, '0', false);
    ui->lineEdit_hw_id_txpart->setText(QString("SI47").append(digistr));

    fw_major = QChar(buff[2]);
    fw_minor = QChar(buff[3]);
    ui->lineEdit_hw_id_txfwrev->setText(QString(fw_major) + "." + QString(fw_minor));

    ui->lineEdit_hw_id_txpatchid->setText(QString::number(buff[4]) + "." + QString::number(buff[5]));

    fw_major = QChar(buff[6]);
    fw_minor = QChar(buff[7]);
    ui->lineEdit_hw_id_txcomprev->setText(QString(fw_major) + "." + QString(fw_minor));

    ui->lineEdit_hw_id_txchiprev->setText(QString(QChar(buff[8])));


    qDebug() << "Chip identified: SI47" << digistr << "-" << QString(QChar(buff[8])) << QString(QChar(buff[2])) << QString(QChar(buff[3])) << "-GM(R)";

    logwrite(LOG_INFO, "CHIP: si47%d, fw=%c.%c, patch_id=0x%02x.0x%02x, "
                       "chip_fw=%c.%c, chip_rev=%c",
             buff[1],
             buff[2],buff[3],
             buff[4],buff[5],
             buff[6],buff[7],
             buff[8]);
    if ((buff[1] == 11)||(buff[1] == 13)){
        if (buff[2] < 2){
            qDebug() << "Chip supports RDS, but firmware: " << QString(QChar(buff[8])) << "." << QString(QChar(buff[2])) << "is NOT, please change your chip to more modern one...";
        } else {
            qDebug() << "Chip supports RDS";
        }
    }

    qDebug() << "Firmware info readed";
    return;
display_firmware_err:
    qDebug() << "Error while reading firmware info";
}

/******************************************************************
 * RDS slots
 ******************************************************************/
void MainWindow::SetRDSPI()
{
    bool ok=false;
    uint16_t u16Val;
    if(!liveEditingEnabled)
        return;
    u16Val=ui->lineEdit_rds_progid->text().toUShort(&ok, 16);
    if(!ok) {
        qDebug() << "Can't set RDS PI: \"" << ui->lineEdit_rds_progid->text() << "\" - bad value";
    } else {
        querier->HidAccessMutex.lock();
        if(fmtxRDSSetPI(u16Val) != FMTX_MODE_OK){
            qDebug() << "Can't set RDS AF";
            SetTransmitIcon(ICON_WARN);
        }
        querier->HidAccessMutex.unlock();
    }
}

void MainWindow::SetRDSPsMiscFlags()
{
    uint16_t MiscFlags = 0;
    if(!liveEditingEnabled)
        return;
    if(ui->checkBox_rds_dynPT->isChecked())
        MiscFlags |= TX_RDS_PS_MISC_RDSD3_MASK;
    if(ui->checkBox_rds_compr_code->isChecked())
        MiscFlags |= TX_RDS_PS_MISC_RDSD2_MASK;
    if(ui->checkBox_rds_art_head->isChecked())
        MiscFlags |= TX_RDS_PS_MISC_RDSD1_MASK;
    if(ui->checkBox_rds_is_stereo->isChecked())
        MiscFlags |= TX_RDS_PS_MISC_RDSD0_MASK;
    if(ui->checkBox_rds_forceb->isChecked())
        MiscFlags |= TX_RDS_PS_MISC_FORCEB_MASK;
    if(ui->radioButton_rds_music->isChecked())
        MiscFlags |= TX_RDS_PS_MISC_RDSMS_MASK;

    MiscFlags |= TX_RDS_PS_MISC_RDSPTY_MASK & (ui->comboBox_rds_pty->currentIndex() << TX_RDS_PS_MISC_RDSPTY_SHFT);

    if(ui->checkBox_rds_ta->isChecked())
        MiscFlags |= TX_RDS_PS_MISC_RDSTA_MASK;
    if(ui->checkBox_rds_ta_tp->isChecked())
        MiscFlags |= TX_RDS_PS_MISC_RDSTP_MASK;
    querier->HidAccessMutex.lock();
    if(fmtxRDSSetPsMiscFlags(MiscFlags) != FMTX_MODE_OK){
        qDebug() << "Can't set RDS Misc flag";
    }
    querier->HidAccessMutex.unlock();
}

void MainWindow::SetRDSAF(double dVal)
{
    if(!liveEditingEnabled)
        return;
    querier->HidAccessMutex.lock();
    if(fmtxRDSSetPsAF(dVal, ui->checkBox_rds_useaf->isChecked()) != FMTX_MODE_OK){
        qDebug() << "Can't set RDS AF";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
}

void MainWindow::toggleRDSaf(int value)
{
    ui->frame_rds_af->setEnabled(value);
    SetRDSAF(ui->doubleSpinBox_rds_af->value());
}

void MainWindow::SetRDSFIFO(int iVal)
{
    if(!liveEditingEnabled)
        return;
    if(ui->checkBox_rds_fifo->isChecked())
        iVal=0;
    querier->HidAccessMutex.lock();
    if(fmtxRDSSetFifoSize(iVal)!= FMTX_MODE_OK){
        qDebug() << "Can't set RDS FIFO";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
}

void MainWindow::toggleRDSfifo(int value)
{
    ui->frame_rds_fifo->setEnabled(value);
    SetRDSFIFO(ui->spinBox_rds_fifo_sz->value());
}

void MainWindow::SetRDSTime(int iVal)
{
    if(!liveEditingEnabled)
        return;
    querier->rdsInterval = iVal * 1000;
}

void MainWindow::toggleRDStime(int value)
{
    ui->frame_rds_time->setEnabled(value);
    if(value) {
        querier->rdsInterval = ui->spinBox_rds_time_int->value() * 1000;
    } else {
        querier->rdsInterval=0;
    }
}

void MainWindow::SetRDSPSNum(int iVal)
{
    if(!liveEditingEnabled)
        return;
    querier->HidAccessMutex.lock();
    if(fmtxRDSSetPsMessageCount(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set RDS PS Number";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
}

void MainWindow::toggleRDSPSNum(int value)
{
    ui->frame_rds_ps_1->setEnabled(value > 0);
    ui->frame_rds_ps_2->setEnabled(value > 1);
    ui->frame_rds_ps_3->setEnabled(value > 2);
    ui->frame_rds_ps_4->setEnabled(value > 3);
    ui->frame_rds_ps_5->setEnabled(value > 4);
    ui->frame_rds_ps_6->setEnabled(value > 5);
    ui->frame_rds_ps_7->setEnabled(value > 6);
    ui->frame_rds_ps_8->setEnabled(value > 7);
    ui->frame_rds_ps_9->setEnabled(value > 8);
    ui->frame_rds_ps_10->setEnabled(value > 9);
    ui->frame_rds_ps_11->setEnabled(value > 10);
    ui->frame_rds_ps_12->setEnabled(value > 11);
    toggleStationId();
    SetRDSPSNum(value);
}

void MainWindow::SetRDSPSPercentage(int iVal)
{
    if(!liveEditingEnabled)
        return;
    querier->HidAccessMutex.lock();
    if(fmtxRDSSetPsMixId(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set RDS PS Percentage";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
}

void MainWindow::SetRDSPSRepeats(int iVal)
{
    if(!liveEditingEnabled)
        return;
    querier->HidAccessMutex.lock();
    if(fmtxRDSSetPsRepeatCount(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set RDS PS Repeats";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
}

int MainWindow::SetRDSPSMessage(int id)
{
    QLineEdit *msg=NULL;
    int ret;
    if(!liveEditingEnabled)
        return FMTX_MODE_NONE;
    msg=GetRDSLineEditById(id);
    querier->HidAccessMutex.lock();
    ret = fmtxRDSSetPsMessageById(id, (char *) msg->text().toStdString().c_str());
    if(ret != FMTX_MODE_OK){
        qDebug() << "Can't set RDS PS Message";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

void MainWindow::toggleRDSRT()
{
    QString text = ui->plainTextEdit_rds_rt->toPlainText();
    SetRDSRT(&text);
}

int MainWindow::SetRDSRT(QString *text)
{
    int ret;
    if(!liveEditingEnabled)
        return FMTX_MODE_NONE;
    querier->HidAccessMutex.lock();
    ret = fmtxRDSSetRtMessage((char *) text->toStdString().c_str());
    if(ret != FMTX_MODE_OK){
        qDebug() << "Can't set RDS RT Message";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

/******************************************************************
 * Transmitter slots
 ******************************************************************/
void MainWindow::SetTransmitPreemphasis(int iVal)
{
    if(!liveEditingEnabled)
        return;
    querier->HidAccessMutex.lock();
    if(fmtxTransmitterSetPreemphasisId(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set transmitter preemphasis flag";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
}

void MainWindow::SetTransmitAudioDeviation(int iVal)
{
    if(!liveEditingEnabled)
        return;
    querier->HidAccessMutex.lock();
    if(fmtxTransmitterSetAudioDeviation(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set transmitter audio deviation flag";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
}

void MainWindow::SetTransmitComponentFlags()
{
    uint8_t ComponentFlags = 0;
    if(!liveEditingEnabled)
        return;
    if(ui->checkBox_transmitter_lmr->isChecked())
        ComponentFlags |= TX_COMPONENT_ENABLE_LMR_MASK;
    if(ui->checkBox_transmitter_pilot->isChecked())
        ComponentFlags |= TX_COMPONENT_ENABLE_PILOT_MASK;
    if(ui->checkBox_transmitter_rds->isChecked())
        ComponentFlags |= TX_COMPONENT_ENABLE_RDS_MASK;
    querier->HidAccessMutex.lock();
    if(fmtxTransmitterSetComponentFlags(ComponentFlags) != FMTX_MODE_OK){
        qDebug() << "Can't set transmitter components flag";
        SetTransmitIcon(ICON_WARN);
    }

    querier->HidAccessMutex.unlock();
}

void MainWindow::SetTransmitPilotDeviation(int iVal)
{
    if(!liveEditingEnabled)
        return;
    querier->HidAccessMutex.lock();
    if(fmtxTransmitterSetPilotDeviation(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set transmitter pilot deviation flag";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
}

void MainWindow::SetTransmitPilotFrequency(int iVal)
{
    if(!liveEditingEnabled)
        return;
    querier->HidAccessMutex.lock();
    if(fmtxTransmitterSetPilotFrequency(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set transmitter pilot frequency flag";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
}

void MainWindow::SetTransmitRDSDeviation(int iVal)
{
    if(!liveEditingEnabled)
        return;
    querier->HidAccessMutex.lock();
    if(fmtxTransmitterSetRDSDeviation(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set transmitter audio deviation flag";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
}

void MainWindow::SetTransmitFrequency(double dVal)
{
    if(!liveEditingEnabled)
        return;
    querier->HidAccessMutex.lock();
    if(fmtxTransmitterSetTuneFreq(dVal) != FMTX_MODE_OK){
        qDebug() << "Can't set transmitter audio deviation flag";
        SetTransmitIcon(ICON_WARN);
    }
    querier->HidAccessMutex.unlock();
}

/* sets power and antcap at the same time */
int MainWindow::SetTransmitPower()
{
    uint8_t ret;
    ui->label_transmitter_level->setText(QString("%1")
                                         .arg(ui->spinBox_transmitter_power->value()));
    if(!liveEditingEnabled)
        return FMTX_MODE_NONE;
    querier->HidAccessMutex.lock();
    if(ui->checkBox_transmitter_antcap->isChecked()) {
        ret=fmtxTransmitterSetTunePower(ui->spinBox_transmitter_power->value(), ui->doubleSpinBox_transmitter_antcap_val->value());
    } else {
        ret=fmtxTransmitterSetTunePower(ui->spinBox_transmitter_power->value(), 0.0);
    }
    if(ret != FMTX_MODE_OK){
        qDebug() << "Can't set transmitter power and capacitor";
        SetTransmitIcon(ICON_WARN);
    } else
    querier->HidAccessMutex.unlock();

    querier->timer.setInterval(2000);
    return ret;
}

void MainWindow::SetTransmitIcon( int iVal )
{
    qDebug() << "SetTransmitIcon" << iVal;
    switch(iVal){
    case ICON_NOTX:
        ui->commandLinkButton_station->setIcon(notxicon);
        trayIcon->setIcon(notxicon);
        ui->actionToggle_transmission->setIcon(notxicon);
        ui->actionToggle_transmission->setText(tr("Enable TX?"));
        break;
    case ICON_TX:
        ui->commandLinkButton_station->setIcon(txicon);
        trayIcon->setIcon(txicon);
        ui->actionToggle_transmission->setIcon(txicon);
        ui->actionToggle_transmission->setText(tr("Disable TX?"));
        break;
    case ICON_WARN:
        ui->commandLinkButton_station->setIcon(warningicon);
        trayIcon->setIcon(warningicon);
        break;
    case ICON_ERR:
        ui->commandLinkButton_station->setIcon(erroricon);
        trayIcon->setIcon(erroricon);
        break;
    case ICON_NODEV:
        ui->commandLinkButton_station->setIcon(nodeviceicon);
        trayIcon->setIcon(nodeviceicon);
        break;
    }
}

void MainWindow::SetTransmitEnabled( bool bEnabled )
{
    uint8_t ret;
    if(!liveEditingEnabled)
        return;

    qDebug() << QString("Transmission: %1").arg(bEnabled?"enabled":"disabled");

    if(bEnabled) {
        ret = SetTransmitPower();
        SetTransmitIcon(ICON_TX);
    } else {
        querier->HidAccessMutex.lock();
        ret=fmtxTransmitterSetTunePower(0, 0.0);
        querier->HidAccessMutex.unlock();
        querier->timer.setInterval(2000);
        SetTransmitIcon(ICON_NOTX);
    }

    if(ret != FMTX_MODE_OK){
        qDebug() << "Can't enable transmission";
        SetTransmitIcon(ICON_WARN);
    }
}

void MainWindow::toggleTransmitEnabled(int value)
{
    ui->frame_transmitter_tx->setEnabled(value);
    ui->verticalSlider_main_power->setEnabled(value);
    SetTransmitEnabled( value );
}

void MainWindow::toggleTransmitAntCap(int value)
{
    ui->frame_transmitter_antcap->setEnabled(value);
    SetTransmitPower();
}

void MainWindow::toggleTransmitRds(int value)
{
    ui->frame_rds_deviation->setEnabled(value);
    SetTransmitComponentFlags();
}

void MainWindow::toggleTransmitPilot(int value)
{
    ui->frame_transmitter_pilot->setEnabled(value);
    SetTransmitComponentFlags();
}

/******************************************************************
 * Audio slots
 ******************************************************************/
void MainWindow::SetAudioCompLimPresetByParams()
{
    if(ui->CheckBox_audio_enablecompressor->isChecked() == false &&
       ui->checkBox_audioLimiterEnable->isChecked() == false) {
        //comp/lim: is disabled
        ui->ComboBox_audioPreset->blockSignals(true);
        ui->ComboBox_audioPreset->setCurrentIndex(0);
        ui->ComboBox_audioPreset->blockSignals(false);
        ui->frame_comp_lim->setEnabled(false);
        qDebug() << "AudioPreset: got disabled";
        return;
    } else if(ui->CheckBox_audio_enablecompressor->isChecked() == false ||
       ui->checkBox_audioLimiterEnable->isChecked() == false) {
        //comp/lim: something - is disabled, custom
        ui->ComboBox_audioPreset->blockSignals(true);
        ui->ComboBox_audioPreset->setCurrentIndex(4);
        ui->ComboBox_audioPreset->blockSignals(false);
        ui->frame_comp_lim->setEnabled(true);
        qDebug() << "AudioPreset: got custom";
        return;
    }

    //comp/lim: both checked, now check values
    if( ui->spinBox_audiocompressorThreshold->value() == -40 &&
            ui->spinBox_audioCompressorGain->value() == 15 &&
            ui->comboBox_audioCompressorReleasetime->currentIndex() == 0/*100*/ &&
            ui->doubleSpinBox_audioCompressorAttackTime->value() == 5.0 &&
            ui->doubleSpinBox_audio_lim_release_time->value() == 5.0196) {
        //comp/lim: minimal
        ui->ComboBox_audioPreset->blockSignals(true);
        ui->ComboBox_audioPreset->setCurrentIndex(1);
        ui->ComboBox_audioPreset->blockSignals(false);
        ui->frame_comp_lim->setEnabled(false);
        qDebug() << "AudioPreset: got minimal";
        return;
    }

    if( ui->spinBox_audiocompressorThreshold->value() == -20 &&
            ui->spinBox_audioCompressorGain->value() == 10 &&
            ui->comboBox_audioCompressorReleasetime->currentIndex() == 2/*350*/ &&
            ui->doubleSpinBox_audioCompressorAttackTime->value() == 2.0 &&
            ui->doubleSpinBox_audio_lim_release_time->value() == 10.0392) {
        //comp/lim: normal
        ui->ComboBox_audioPreset->blockSignals(true);
        ui->ComboBox_audioPreset->setCurrentIndex(2);
        ui->ComboBox_audioPreset->blockSignals(false);
        ui->frame_comp_lim->setEnabled(false);
        qDebug() << "AudioPreset: got normal";
        return;
    }

    if( ui->spinBox_audiocompressorThreshold->value() == -15 &&
            ui->spinBox_audioCompressorGain->value() == 5 &&
            ui->comboBox_audioCompressorReleasetime->currentIndex() == 4/*1000*/ &&
            ui->doubleSpinBox_audioCompressorAttackTime->value() == 0.5 &&
            ui->doubleSpinBox_audio_lim_release_time->value() == 51.2) {
        //comp/lim: normal
        ui->ComboBox_audioPreset->blockSignals(true);
        ui->ComboBox_audioPreset->setCurrentIndex(3);
        ui->ComboBox_audioPreset->blockSignals(false);
        ui->frame_comp_lim->setEnabled(false);
        qDebug() << "AudioPreset: got normal";
        return;
    }

    //comp/lim: unknown, set custom
    ui->ComboBox_audioPreset->blockSignals(true);
    ui->ComboBox_audioPreset->setCurrentIndex(4);
    ui->ComboBox_audioPreset->blockSignals(false);
    ui->frame_comp_lim->setEnabled(true);
    qDebug() << "AudioPreset: got unknown:"
             << " CT:" << ui->spinBox_audiocompressorThreshold->value()
             << " CG:" << ui->spinBox_audioCompressorGain->value()
             << " RT:" << ui->comboBox_audioCompressorReleasetime->currentIndex()
             << " AT:" << ui->doubleSpinBox_audioCompressorAttackTime->value()
             << " LR:" << ui->doubleSpinBox_audio_lim_release_time->value();

    return;

}

void MainWindow::SetAudioCompLimByPreset(int iVal)
{
    /*0 - dis, 1 - minimal, 2 - normal, 3 - agressive, 4 - custom */
    if(iVal==4) {
        ui->frame_comp_lim->setEnabled(true);
    } else {
        ui->frame_comp_lim->setEnabled(false);
    }

    switch(iVal){
    case 0:
        ui->CheckBox_audio_enablecompressor->setChecked(false);
        ui->checkBox_audioLimiterEnable->setChecked(false);
        break;
    case 1:
        ui->CheckBox_audio_enablecompressor->setChecked(true);
        ui->spinBox_audiocompressorThreshold->setValue(-40);
        ui->spinBox_audioCompressorGain->setValue(15);
        ui->comboBox_audioCompressorReleasetime->setCurrentIndex(0/*100*/);
        ui->doubleSpinBox_audioCompressorAttackTime->setValue(5.0);
        ui->checkBox_audioLimiterEnable->setChecked(true);
        ui->doubleSpinBox_audio_lim_release_time->setValue(5.0196);
        break;
    case 2:
        ui->CheckBox_audio_enablecompressor->setChecked(true);
        ui->spinBox_audiocompressorThreshold->setValue(-20);
        ui->spinBox_audioCompressorGain->setValue(10);
        ui->comboBox_audioCompressorReleasetime->setCurrentIndex(2/*350*/);
        ui->doubleSpinBox_audioCompressorAttackTime->setValue(2.0);
        ui->checkBox_audioLimiterEnable->setChecked(true);
        ui->doubleSpinBox_audio_lim_release_time->setValue(10.0392);
        break;
    case 3:
        ui->CheckBox_audio_enablecompressor->setChecked(true);
        ui->spinBox_audiocompressorThreshold->setValue(-15);
        ui->spinBox_audioCompressorGain->setValue(5);
        ui->comboBox_audioCompressorReleasetime->setCurrentIndex(4/*1000*/);
        ui->doubleSpinBox_audioCompressorAttackTime->setValue(0.5);
        ui->checkBox_audioLimiterEnable->setChecked(true);
        ui->doubleSpinBox_audio_lim_release_time->setValue(51.2);
        break;
    }
}

bool MainWindow::SetAudioCompLimFlags()
{
    uint8_t CompLimFlags = 0;
    bool ret = true;
    if(!liveEditingEnabled)
        return false;
    if(ui->CheckBox_audio_enablecompressor->isChecked())
        CompLimFlags |= TX_ACOMP_ENABLE_ACEN_MASK;
    if(ui->checkBox_audioLimiterEnable->isChecked())
        CompLimFlags |= TX_ACOMP_ENABLE_LIMITEN_MASK;
    querier->HidAccessMutex.lock();
    if(fmtxAudioSetAcompFlags(CompLimFlags) != FMTX_MODE_OK){
        qDebug() << "Can't set compressor/limiter flag";
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

void MainWindow::toggleCompressor(int value)
{
    ui->frame_audio_acomp->setEnabled(value);
    SetAudioCompLimFlags();
}

bool MainWindow::SetAudioCompressorThreshold(int iVal)
{
    bool ret = true;
    if(!liveEditingEnabled)
        return false;
    querier->HidAccessMutex.lock();
    if(fmtxAudioSetAcompThreshold(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set audio compressor threshold";
        SetTransmitIcon(ICON_WARN);
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

bool MainWindow::SetAudioCompressorGain(int iVal)
{
    bool ret = true;
    if(!liveEditingEnabled)
        return false;
    querier->HidAccessMutex.lock();
    if(fmtxAudioSetAcompGain(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set audio compressor gain";
        SetTransmitIcon(ICON_WARN);
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

bool MainWindow::SetAudioCompressorReleaseTime(int iVal)
{
    bool ret = true;
    if(!liveEditingEnabled)
        return false;
    querier->HidAccessMutex.lock();
    if(fmtxAudioSetAcompReleaseTimeId(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set audio compressor release time";
        SetTransmitIcon(ICON_WARN);
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

bool MainWindow::SetAudioCompressorAttackTime(double dVal)
{
    bool ret = true;
    if(!liveEditingEnabled)
        return false;
    querier->HidAccessMutex.lock();
    if(fmtxAudioSetAcompAttackTime(dVal) != FMTX_MODE_OK){
        qDebug() << "Can't set audio compressor attack time";
        SetTransmitIcon(ICON_WARN);
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

void MainWindow::toggleLimiter(int value)
{
    ui->frame_audio_alim->setEnabled(value);
    SetAudioCompLimFlags();
}

bool MainWindow::SetAudioLimiterReleaseTime(double dVal)
{
    bool ret = true;
    int new_div;

    //prevent division by zero
    if(dVal < 0.256)
        dVal = 0.256;

    new_div = 512.0/dVal;
    //take care about 5th digitsa after point that cutted
    if((audioLimiter_prev_div == new_div) &&
            (512.0/audioLimiter_prev_div - dVal > 0.0001))
        new_div++;

    if(new_div < 5)
        new_div=5;
    if(new_div > 2000)
        new_div=2000;

    audioLimiter_prev_div = new_div;
    qDebug() << __FUNCTION__ << "set_div: " << audioLimiter_prev_div
             << " val:" << 512.0/audioLimiter_prev_div;
    ui->doubleSpinBox_audio_lim_release_time->blockSignals(true);
    ui->doubleSpinBox_audio_lim_release_time->setValue(512.0/audioLimiter_prev_div);
    ui->doubleSpinBox_audio_lim_release_time->blockSignals(false);

    if(!liveEditingEnabled)
        return false;
    querier->HidAccessMutex.lock();
    if(fmtxAudioSetLimiterReleaseTime(512.0/audioLimiter_prev_div) != FMTX_MODE_OK){
        qDebug() << "Can't set audio limiter release time";
        SetTransmitIcon(ICON_WARN);
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

bool MainWindow::SetAudioASQFlags()
{
    bool ret = true;
    uint8_t ASQFlags = 0;
    if(!liveEditingEnabled)
        return false;
    if(ui->checkBox_asq_overmoddetect->isChecked())
        ASQFlags |= TX_ASQ_INT_SELECT_OVERMODIEN_MASK;
    if(ui->checkBox_asq_detectLow->isChecked())
        ASQFlags |= TX_ASQ_INT_SELECT_IALLIEN_MASK;
    if(ui->checkBox_asq_detectHigh->isChecked())
        ASQFlags |= TX_ASQ_INT_SELECT_IALHIEN_MASK;
    querier->HidAccessMutex.lock();
    if(fmtxAudiosetAsqIntFlags(ASQFlags) != FMTX_MODE_OK){
        qDebug() << "Can't set ASQ flags";
        SetTransmitIcon(ICON_WARN);
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

bool MainWindow::SetAudioASQLevelLow(int iVal)
{
    bool ret = true;
    if(!liveEditingEnabled)
        return false;
    querier->HidAccessMutex.lock();
    if(fmtxAudioSetAsqLevelLow(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set ASQ Low Level Threshold";
        SetTransmitIcon(ICON_WARN);
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

bool MainWindow::SetAudioASQDurationLow(int iVal)
{
    bool ret = true;
    if(!liveEditingEnabled)
        return false;
    querier->HidAccessMutex.lock();
    if(fmtxAudioSetAsqDurationLow(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set ASQ Low Duration";
        SetTransmitIcon(ICON_WARN);
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

bool MainWindow::SetAudioASQLevelHigh(int iVal)
{
    bool ret = true;
    if(!liveEditingEnabled)
        return false;
    querier->HidAccessMutex.lock();
    if(fmtxAudioSetAsqLevelHigh(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set ASQ High Level Threshold";
        SetTransmitIcon(ICON_WARN);
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

bool MainWindow::SetAudioASQDurationHigh(int iVal)
{
    bool ret = true;
    if(!liveEditingEnabled)
        return false;
    querier->HidAccessMutex.lock();
    if(fmtxAudioSetAsqDurationHigh(iVal) != FMTX_MODE_OK){
        qDebug() << "Can't set ASQ High Duration";
        SetTransmitIcon(ICON_WARN);
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}

void MainWindow::toggleASQLow(int value)
{
    ui->frame_audio_asq_low->setEnabled(value);
    SetAudioASQFlags();
}

void MainWindow::toggleASQHigh(int value)
{
    ui->frame_audio_asq_high->setEnabled(value);
    SetAudioASQFlags();
}



bool MainWindow::SetDigitalAudioHack()
{
    bool ret = true;
    if(!liveEditingEnabled)
        return false;
    querier->HidAccessMutex.lock();
    if(fmtxIoAppEnableAudio() != FMTX_MODE_OK){
        qDebug() << "Can't set Digital audio hack";
        SetTransmitIcon(ICON_WARN);
        ret = false;
    }
    querier->HidAccessMutex.unlock();
    return ret;
}
